
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2011 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
# PROGRAM   oppmaker.pl 
# VERSION   1.0 
# DATE      23/02/2001, 23/07/2008
# PURPOSE   generate automatically some parser options in atl.opp for any compiler 

use File::Copy;
use File::Basename;

$SCRIPTDIR="$ENV{'TESTRTDIR'}/lib/scripts/";
$INTDIR=".";
$TARGETDIR=$ENV{'ATLTGT'};
#
# ATLTGT must be set to the TDP before using oppmaker
die "ATLTGT does not seem to be correctly set.\nPlease Fix It first!\n" if(! -d "$TARGETDIR/cmd");
die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
my @GENERATION_CONDITION;
my @COMPLETE_INCLUDE_DIR_LIST;

require "$TARGETDIR/cmd/atl_cc.pl";
require "$TARGETDIR/cmd/atl_link.pl";
require "$TARGETDIR/cmd/atl_func.pl";
require $SCRIPTDIR . "Fcts.pl";
$BINDIR=GetBinDir();

#
# Getting sytem includes
#
sub test_CC_sys_include
{
	my $file=shift;
	my $cmp=shift;
	my $opt=shift;
	my $index=0;

	$out=basename($file,".$Ini{'SRCEXT'}").".i";
	$ret=atl_cpp("CPP",basename($file),$out,$Ini{'DEFAULT_CPPFLAGS'},\@GENERATION_CONDITION,\@COMPLETE_INCLUDE_DIR_LIST);
	die "Preprocessing Error on $file\n" if($ret!=0);
	open(FTMP,"<$out") or die "unable to open $out";
	while(<FTMP>)
	{
		if (/#/ && (/\// || /\\/))
		{
			my ($dp,$bn,$e) = ($_ =~ /^((?:.*[:\\\/])?)(.*)\.(.*)/);
			if (length $dp>0)
			{
				@tmp = split /"/, $dp; #"
				my $flag=0;
				foreach $tab (@tab)
				{
					if ($tab eq $tmp[1])
					{
						$flag=1;
					}
				}
				if ($flag==0)
				{
					$tab[$index] = $tmp[1];
					$index++;
					if (-f $tmp[1]."typeinfo.h")
					{
						$IsTypeinfo=1;
					}
				}
			}
		}
	}
	@tab2 = sort @tab;
	foreach $tab2 (@tab2)
	{
		$tab2 =~ s/\\\\/\\/g;
		$tab2 =~ s/\/\//\//g;
		chop $tab2;
		print FOPT "$opt=$tab2\n";
		print  "$opt=$tab2\n";
	}
	close(FTMP);
	#  unlink($basename.i");
}

#
# Getting stuff for compilation
#
sub test_CPP
{
	my $file=shift;
	my $cmp=shift;
	my $opt=shift;

	$out=basename($file,".$Ini{'SRCEXT'}").".i";
	$ret=atl_cpp("CPP",basename($file),$out,$Ini{'DEFAULT_CPPFLAGS'},\@GENERATION_CONDITION,\@COMPLETE_INCLUDE_DIR_LIST);
	open(FTMP, "<$out") or die "unable to open $out";
	while(<FTMP>)
	{
		next if (/#/ && /\\\\/);
		print FOPT $opt."\n" if (/#/ && (/\\/));
		print  $opt."\n" if (/#/ && (/\\/));
	}
	close(FTMP);
}

#
# Getting stuff for compilation
#
sub test_CC
{
	my $file=shift;
	my $cmp=shift;
	my $opt=shift;

	$out=basename($file,".$Ini{'SRCEXT'}").".$Ini{'OBJEXT'}";
	$ret=atl_cc("CPP",$file,$out,"$Ini{'DEFAULT_CPPFLAGS'} $Ini{'DEFAULT_CFLAGS'}",\@GENERATION_CONDITION,\@COMPLETE_INCLUDE_DIR_LIST);
	if ((($ret==0)&&($cmp==1)) || (($ret!=0)&&($cmp==0)))
	{
		print FOPT $opt."\n";
		print  $opt."\n";
	}
}

#
# Main
#
my $ret;

#
# No Output Bufferisation
#
$|=1;

#
# Delivered default C++ source extension and source files
#
my $CppOrigExt="cpp";
my %Options;

my @OrigSourceFiles=qw(a0.cpp a1.cpp a2.cpp a3.cpp a4.cpp a5.cpp a6.cpp a7.cpp a8.cpp a9.cpp 
		a10.cpp a11.cpp a12.cpp a13.cpp a14.cpp a15.cpp a16.cpp a17.cpp a19.cpp 
		a20.cpp a21.cpp a22.cpp a23.cpp a24.cpp a25.cpp 
		a10a.cpp);
my @SourceFiles=@OrigSourceFiles;

#
# Read the tpcpp.ini to know the C++ extension and be able to run atl_cc and others
#
$INIFILE="tpcpp.ini";
$ret=DecodeIniFile();
die "Cannot open $ENV{'ATLTGT'}/$INIFILE\n" if($ret==0);

#
# Rename the delivered source files into the wanted ones
#
if($Ini{'SRCEXT'} ne $CppOrigExt)
{
	foreach(@OrigSourceFiles)
	{
		my $newName=basename($_,".$CppOrigExt").".$Ini{'SRCEXT'}";
		copy($_,$newName) || die "cannot copy $_ into $newName\n";
		push(@SourceFiles,$newName);
	}
}

$IsTypeinfo=0;
foreach (split(/,/,$Ini{'DEFAULT_DEFINES'}))
{
	push(@GENERATION_CONDITION,$_);
}
foreach (split(/,/,$Ini{'DEFAULT_INCLUDE_PATHS'}))
{
	push(@COMPLETE_INCLUDE_DIR_LIST,$_);
}
open(FOPT, ">atl.opp") or die "unable to create atl.opp";

#foreach(@SourceFiles)
#{
#	print $_."\n";
#}

test_CC_sys_include("a0.cpp",0,"--sys_include");
test_CC("a1.$Ini{'SRCEXT'}",0,"--no_explicit");
test_CC("a2.$Ini{'SRCEXT'}",1,"--old_for_init");
test_CC("a3.$Ini{'SRCEXT'}",1,"--dollar");
test_CC("a4.$Ini{'SRCEXT'}",0,"--no_restrict");
test_CC("a5.$Ini{'SRCEXT'}",0,"--no_wchar_t_keyword");
test_CC("a6.$Ini{'SRCEXT'}",0,"--no_bool");
test_CC("a7.$Ini{'SRCEXT'}",0,"--no_typename");
test_CC("a8.$Ini{'SRCEXT'}",1,"--special_subscript_cost");
test_CC("a9.$Ini{'SRCEXT'}",0,"--no_alternative_tokens");
test_CC("a10.$Ini{'SRCEXT'}",1,"--microsoft_asm");
test_CC("a10a.$Ini{'SRCEXT'}",1,"--microsoft_union_with_array");
test_CC("a11.$Ini{'SRCEXT'}",1,"--nonconst_ref_anachronism");
test_CC("a12.$Ini{'SRCEXT'}",1,"--no_const_string_literal");
test_CC("a13.$Ini{'SRCEXT'}",0,"--no_class_name_injection");
test_CC("a14.$Ini{'SRCEXT'}",1,"--nonstd_using_decl");
test_CC("a15.$Ini{'SRCEXT'}",1,"--variadic_macros");
test_CC("a16.$Ini{'SRCEXT'}",1,"--extended_variadic_macros");
test_CC("a17.$Ini{'SRCEXT'}",1,"--diag_suppress 983");
test_CC("a19.$Ini{'SRCEXT'}",1,"--guiding_decls");
test_CC("a20.$Ini{'SRCEXT'}",1,"--old_specializations");
test_CC("a21.$Ini{'SRCEXT'}",1,"--void_star_null_pointer");
if ($IsTypeinfo == 1) {
   test_CC("a22.$Ini{'SRCEXT'}",1,"--no_type_info_in_namespace_std");
}
test_CC("a23.$Ini{'SRCEXT'}",1,"--gnu_mode");
test_CC("a24.$Ini{'SRCEXT'}",1,"--ignore_std");
test_CPP("a25.$Ini{'SRCEXT'}",1,"--old_style_preprocessing");

close(FOPT);
print "\n*\n*\n* atl.opp in this directory";
print " contains options to be set into your TDP Parser Settings > Analyzer file configuration\n*\n*\n";

